unit CyDRecorder;

interface

uses
  mmsystem, Classes, windows, Math, Sysutils;

type
  TSoundRecorder = class(TThread)
  private
    { Private declarations }
    procedure Process;
    procedure UpdateFile(hFile:THandle;m_RecordPosition:Integer);
    procedure CloseWaveRec(hEvent:THandle);
    procedure WaveInCheck(res:MMResult);
  protected
    procedure Execute; override;
  public
    hRecord:HWAVEIN;
    hEvent:THandle;
  end;

implementation

uses SoundMain;

{ TSoundRecorder }

procedure TSoundRecorder.WaveInCheck(res:MMResult);
var
 ErrText:array [0..MAXERRORLENGTH+1] of char;
begin
 if res=MMSYSERR_NOERROR then exit;
 Terminate;
 waveInGetErrorText(res,ErrText,Sizeof(ErrText));
 MessageBox(SounderForm.Handle,ErrText, 'Error', MB_OK+MB_ICONEXCLAMATION);
 Free;
end;

procedure TSoundRecorder.CloseWaveRec(hEvent:THandle);
begin
  MessageBox(SounderForm.Handle,'Wave in error', 'Error', MB_OK+MB_ICONEXCLAMATION);
  CloseHandle(hEvent);
  waveInClose(hRecord);
end;

procedure TSoundRecorder.Execute;
begin
 Process;
end;

procedure TSoundRecorder.Process;
const DEFAULT_BLOCKSIZE=64*1024;
var
 si:TSystemInfo;
 BlockSize, ii:Integer;
 curhdr:WAVEHDR;
 BytesWritten:DWord;
 Header:array [0..2] of WAVEHDR;
 hFile:THANDLE;
 shift:Integer;
 m_RecordPosition:Int64;
begin
 BlockSize:=DEFAULT_BLOCKSIZE;
 GetSystemInfo(si);
 ZeroMemory(@Header,sizeof(Header));
 hEvent:=CreateEvent(nil,true,false,nil);

  WaveInCheck(waveInOpen(@hRecord,SounderForm.w_DeviceID, @SounderForm.wfx, hEvent,0,CALLBACK_EVENT));
  hFile:=CreateFile(PChar(SounderForm.WavFileName),GENERIC_WRITE,0,nil,OPEN_ALWAYS, FILE_ATTRIBUTE_ARCHIVE,0);
  if hFile=INVALID_HANDLE_VALUE then
   begin
    CloseWaveRec(hEvent);
    Terminate;
    Free;
    exit;
   end;
  Header[0].lpData:=VirtualAlloc(nil,round((BlockSize*2+Integer(si.dwPageSize)-1)/si.dwPageSize*si.dwPageSize),MEM_RESERVE+MEM_COMMIT, PAGE_READWRITE);
  if Header[0].lpData=nil then
   begin
    CloseWaveRec(hEvent);
    Terminate;
    Free;
    exit;
   end;
  Header[1].lpData:=Header[0].lpData+BlockSize;
  Header[0].dwBufferLength:=BlockSize;
  Header[1].dwBufferLength:=BlockSize;

  waveInCheck(waveInPrepareHeader(hRecord,@Header[0],sizeof(waveHDR)));
  waveInCheck(waveInPrepareHeader(hRecord,@Header[1],sizeof(waveHDR)));
  SetFilePointer(hFile,SounderForm.DataOffset,nil,FILE_BEGIN);

  waveInCheck(waveInAddBuffer(hRecord,@Header[0],sizeof(waveHDR)));
  waveInCheck(waveInAddBuffer(hRecord,@Header[1],sizeof(waveHDR)));

  ResetEvent(hEvent);
  waveInCheck(waveInStart(hRecord));
  ii:=0;

  m_RecordPosition:=SounderForm.w_RestartPosition+SounderForm.DataOffset;
  SetFilePointer(hFile,m_RecordPosition,nil,FILE_BEGIN);

  while SounderForm.w_Stop<>true do
   begin
    WaitForSingleObject(hEvent,INFINITE);
    ResetEvent(hEvent);
    curhdr:=Header[ii and 1];
    Inc(ii);
    if curhdr.dwBytesRecorded>0 then
     begin
      WriteFile(hFile, curHdr.lpData^,curHdr.dwBytesRecorded, BytesWritten, nil);
      m_RecordPosition:=m_RecordPosition+Integer(curHdr.dwBytesRecorded);
      curHdr.dwBytesRecorded:=0;
     end;
    if SounderForm.w_Stop<>true then
     waveInAddBuffer(hRecord, @curHdr, sizeof(WAVEHDR));
   end;

  ResetEvent(hEvent);
  waveInStop(hRecord);
  waveInReset(hRecord);
  waveInCheck(waveInUnprepareheader(hRecord,@Header[0],sizeof(waveHDR)));
  waveInCheck(waveInUnprepareheader(hRecord,@Header[1],sizeof(waveHDR)));
  waveInClose(hRecord);
  hRecord:=0;
  if hFile<>INVALID_HANDLE_VALUE then
   UpdateFile(hFile,m_RecordPosition);
  CloseHandle(hFile);
  CloseHandle(hEvent);
  Synchronize(SounderForm.SaveRecFile);
  Terminate;
end;

procedure TSoundRecorder.UpdateFile(hFile: THandle;
  m_RecordPosition: Integer);
var
 BytesRecorded,Data:DWord;
begin
 SetFilePointer(hFile,4,nil,FILE_BEGIN);

 if m_RecordPosition>SounderForm.DataSize then
  Data:=m_RecordPosition+sizeof(PCMWAVEFORMAT)+20
 else
  Data:=SounderForm.DataSize+sizeof(PCMWAVEFORMAT)+20;
 WriteFile(hFile,Data,4, BytesRecorded,nil);
 SetFilePointer(hFile,24+sizeof(PCMWAVEFORMAT),nil,FILE_BEGIN);

 if m_RecordPosition>SounderForm.DataSize then
  WriteFile(hFile,m_RecordPosition,4, BytesRecorded,nil)
 else
  WriteFile(hFile,SounderForm.DataSize,4, BytesRecorded,nil);

 if m_RecordPosition>SounderForm.DataSize then
  SetFilePointer(hFile,m_RecordPosition,nil,FILE_CURRENT)
 else
  SetFilePointer(hFile,SounderForm.DataSize,nil,FILE_CURRENT);
 SetEndOfFile(hFile);
end;

end.
